package infosys;

import java.sql.*;
import de.web.databasesupport.tools.database.*;

/** Logic implementation for checking in one freight.
 *  To use this class register an instance in the Dispatcher
 *  for CheckInTask.
 */
public class CheckInLogic implements Logic {

  protected Database db = null;

  public CheckInLogic(Database db) {
      this.db = db;
  }

  /** SQL String to select the catalogId of the freight.
     *  String is used as a PreparedStatement with 1 parameter.
     *  Set the parameters in following order:<br>
     *  name : String
     */
  protected String str_query_catalog =
  "SELECT catalogId FROM Catalog_Freight WHERE name = ?;";

    /** SQL String to insert a catalog name.
     *  String is used as a PreparedStatement with 1 parameter.
     *  Set the parameters in following order:<br>
     *  name : String
     */
  protected String str_insert_catalog =
  "INSERT INTO Catalog_Freight (name) VALUES (?);";

  /** SQL String to get all free x, y positions. */
  protected String str_query_board =
  "SELECT posX, posY FROM Board WHERE paletteId IS NULL;";

  /** SQL String retrieving all statistic data for this type of freight.
     *  String is used as a PreparedStatement with 1 parameter.
     *  Set the parameters in following order:<br>
     *  cataloId : int
     */
  protected String str_query_frequency =
  "SELECT countIn, countOut, rampId FROM Frequency WHERE catalogId = ?";

    /** SQL String inserting a freight into the db.
     *  String is used as a PreparedStatement with 6 parameters.
     *  Set the parameters in following order:<br>
     *  catalogId  : int<br>
     *  checkIn    : java.sql.Date<br>
     *  expiration : java.sql.Date<br>
     *  weight     : int<br>
     *  height     : int<br>
     *  amount     : int<br>
     */
  protected String str_insert_freight =
  "INSERT INTO Freight (catalogId, checkIn, expiration, weight, height, amount) VALUES(?,?,?,?,?,?);";

    /** SQL String inserting a palette into the db.
     *  String is used as a PreparedStatement with 2 parameters.
     *  Set the parameters in following order:<br>
     *  weight     : int<br>
     *  height     : int<br>
     */
  protected String str_insert_palette =
  "INSERT INTO Palette (width, height) VALUES (?,?);";

    /** SQL String inserting a box into the db.
     *  String is used as a PreparedStatement with 5 parameters.
     *  Set the parameters in following order:<br>
     *  paletteId  : int<br>
     *  freightId  : java.sql.Date<br>
     *  ix         : int<br>
     *  iy         : int<br>
     *  iz         : int<br>
     */
  protected String str_insert_box     =
  "INSERT INTO Box (palettenId, freightId, ix, iy, iz) VALUES (?,?,?,?,?);";

    /** SQL String updating a palette.
     *  String is used as a PreparedStatement with 3 parameters.
     *  Set the parameters in following order:<br>
     *  paletteId : int<br>
     *  posX      : int<br>
     *  posY      : int<br>
     */
  protected String str_update_board =
  "UPDATE Board SET paletteId = ? WHERE posX = ? AND posY = ?;";

    /** SQL String updating statistic data (frequency).
     *  String is used as a PreparedStatement with 3 parameters.
     *  Set the parameters in following order:<br>
     *  countIn   : int<br>
     *  rampId    : int<br>
     *  catalogID : int<br>
     */
  protected String str_update_frequency =
  "UPDATE Frequency SET countIn = ? WHERE rampId = ? AND catalogId = ?;";

    /** PreparedStatement. @see #str_query_catalog */
  protected PreparedStatement pre_query_catalog    = null;
    /** PreparedStatement. @see #str_insert_catalog */
  protected PreparedStatement pre_insert_catalog   = null;
    /** PreparedStatement. @see #str_query_board */
  protected PreparedStatement pre_query_board      = null;
    /** PreparedStatement. @see #str_query_frequency */
  protected PreparedStatement pre_query_frequency  = null;
    /** PreparedStatement. @see #str_insert_freight */
  protected PreparedStatement pre_insert_freight   = null;
    /** PreparedStatement. @see #str_insert_palette */
  protected PreparedStatement pre_insert_palette   = null;
    /** PreparedStatement. @see #str_insert_box */
  protected PreparedStatement pre_insert_box       = null;
    /** PreparedStatement. @see #str_update_board */
  protected PreparedStatement pre_update_board     = null;
    /** PreparedStatement. @see #str_update_frequency */
  protected PreparedStatement pre_update_frequency = null;

    /** Initializes the SQL-Statements.
     *  NOT IMPLEMENTED YET
     */
    private void initSQL() {
    throw new UnsupportedOperationException("initSQL() n.i.");
    }

  /** Processes the task.
   *  @todo IMPLEMENT THIS FEATURE
   *  @param task The CheckInTask to process.
   *  @throw ClassCastException when task is not an instance of CheckInTask
   */
  public void process(Task task) {
    CheckInTask in = (CheckInTask) task;
    task.fireTaskProcessed("Task is beeing processed by CheckInLogic");
    int strategy = MainFrame.strategy;
    int strategyIn = strategy & MainFrame.STRATEGY_IN_MASC;
    int strategyTower = strategy & MainFrame.STRATEGY_TOWER_MASC;
    Control.Unit ramp = (Control.Unit) in.source;

    int rampPosX = ramp.getX();
    int rampPosY = ramp.getY();

    /** @todo a stationNr must be an int (1 or 2) coded
     *  in the driver from wenkman
     *  stationNr = 1 is pos(1,1)
     *  stationNr = 2 is pos(10,10)*/
    int stationNr = 1;// test

    // position where to put the freigt in. Generated by the strategy.
    int posX;
    int posY;

    switch (strategyIn) {
      case MainFrame.STRATEGY_IN_RANDOM: {
              task.fireTaskProcessed("Using random CheckIn-Strategy");
              // implement a checkIn-Strategy using a randomly place
              // for the freight.
              java.util.Vector freePos = getFreeBoardPositions();
              java.util.Random r = new java.util.Random();
              int randomPos = r.nextInt(freePos.size());
              Object choosen = freePos.get(randomPos);
              if (choosen instanceof java.awt.Point) {
                java.awt.Point pos = (java.awt.Point) choosen;
                posX = pos.getX();
                posY = pos.getY();
              }
              break;
      }
      case MainFrame.STRATEGY_IN_FASTEST_FIT: {
              task.fireTaskProcessed("Using fastest fit CheckIn-Strategy");
              // implement a checkIn-Strategy that uses the nearest place
              // for the freight from the actuall position.
              // get nearest free position from the checkIn-ramp.
              break;
      }
      case MainFrame.STRATEGY_IN_STATISTICAL: {
              task.fireTaskProcessed("Using statistical CheckIn-Strategy");
              // implement a checkIn-Strategy using statistical information
              // for a ideal place of the freight
              break;
      }
      default: {
              task.fireTaskTerminated("There was no strategy specified");
              return;
      }

      /** @todo call driver to move with posX and posY as destination*/
      // long errorCode = hrregalDriver.einlagern(posX,posY,station);
      // if (errorCode != 0) {
          //there was an error
      // }

    }
    switch(strategyTower) {
      case MainFrame.STRATEGY_TOWER_LAST_POS: {
              task.fireTaskProcessed("Tower stays at last position");
              // do nothing... tower stays where it is
              break;
      }
      case MainFrame.STRATEGY_TOWER_FAVORITE_POS: {
              task.fireTaskProcessed("Tower moves to a strategic position");
              // move the tower to a stragical position using frequency-Count.
              // dummy
              int towerX = 5;
              int towerY = 1;
              //long errorCode = hregalDriver.positionieren(towerX,towerY);
              // if (errorCode != 0) {
              //    there was an error
              // }
              break;
      }
    }
    /** @todo IMPLEMENT THIS FEATURE */

    task.fireTaskTerminated("after a commit and when the driver responded the task is terminated");
  }

  /** Gets all free board positions and puts them as Points into
   *  the returning Vector.
   *  @return Returns a java.util.Vector of java.awt.Point - objects with the
   *  free positions.
   */
  public Vector getFreeBoardPositions() {
    ResultSet result = null;
    java.util.Vector free = new java.util.Vector();
    int posX;
    int posY;
/*
    result = pre_query_board.executeQuery();
    while (result.next()) {
      posX = result.getInt(1);
      posY = result.getInt(2);
      java.awt.Point p = new java.awt.Point(posX,posY);
      free.add(p);
    }
*/
    return free;
  }

}// end of class