package infosys;

/** This is a special Task for checking in freights from a Unit (checkIn).
 *  The CheckInLogic is responsible for handling this kind of tasks.
 */
public class CheckInTask extends Task {

  /** The name of the freight. Must be the name of the DB catalog..
   *  If the name doesn't exist as an catalog entry is inserted.
   */
  protected String freightName;

  /** The amount of the freight to check in. */
  protected long amount;

  /** The date of expiration. for this freight. */
  protected java.sql.Date expiration;

  /** Constructs a new CheckInTask for this freight.
   *  @param freightName The catalog name of the freight.
   *  @param amount The amount to check in.
   *  @param expiration The date of expiration.
   */
  public CheckInTask (Object source, String freightName, long amount, java.sql.Date expiration) {
    this.source = source;
    this.freightName = freightName;
    this.amount = amount;
    this.expiration = expiration;
  }

  /** Returns the catalog name for this freight.
   *  @return The catalog name of the freight.
   */
  public String getFreightName() {
    return this.freightName;
  }

  /** Returns the amount for this freight.
   *  @return The amount.
   */
  public long getAmount() {
    return this.amount;
  }

  /** The date of expiration.
   *  @return The date of expiration.
   */
  public java.sql.Date getExpiration() {
    return this.expiration;
  }

  public String toString() {
    return "Check in : " + amount + " " + freightName + " expiration is " + expiration;
  }

} // end of class