package infosys;

import java.sql.*;
import de.web.databasesupport.tools.database.*;

/** Logic implementation for checking out one freight.
 *  To use this class register an instance in the Dispatcher
 *  for CheckOutTask.
 */
public class CheckOutLogic implements Logic {

  protected Database db = null;

  public CheckOutLogic(Database db) {
      this.db = db;
  }

  /** SQL String to select the catalogId of the freight.
     *  String is used as a PreparedStatement with 1 parameter.
     *  Set the parameters in following order:<br>
     *  name : String
     */
  protected String str_query_catalog =
  "SELECT catalogId FROM Catalog_Freight WHERE name = ?;";

    /** SQL String updating statistic data (frequency).
     *  String is used as a PreparedStatement with 3 parameters.
     *  Set the parameters in following order:<br>
     *  countOut  : int<br>
     *  rampId    : int<br>
     *  catalogID : int<br>
     */
  protected String str_update_frequency =
  "UPDATE Frequency SET countOut = ? WHERE rampId = ? AND catalogId = ?;";

    /** PreparedStatement. @see #str_query_catalog */
  protected PreparedStatement pre_query_catalog    = null;
    /** PreparedStatement. @see #str_update_frequency */
  protected PreparedStatement pre_update_frequency = null;

    /** Initializes the SQL-Statements.
     *  NOT IMPLEMENTED YET
     */
    private void initSQL() {
    throw new UnsupportedOperationException("initSQL() n.i.");
    }

  /** Processes the task.
   *  @todo IMPLEMENT THIS FEATURE
   *  @param task The CheckOutTask to process.
   *  @throw ClassCastException when task is not an instance of CheckOutTask
   */
  public void process(Task task) {
    CheckOutTask in = (CheckOutTask) task;
    task.fireTaskProcessed("Task is beeing processed by CheckOutLogic");
    int strategy = MainFrame.strategy;
    int strategyOut = strategy & MainFrame.STRATEGY_OUT_MASC;
    int strategyTower = strategy & MainFrame.STRATEGY_TOWER_MASC;

    switch (strategyOut) {
      case MainFrame.STRATEGY_OUT_EXPIRATION: {
              task.fireTaskProcessed("Using expiration CheckOut-Strategy");
              // implement a CheckOut-Strategy by using the expiration of
              // the freight as criterium.
              break;
      }
      case MainFrame.STRATEGY_OUT_FIRST_FIT: {
              task.fireTaskProcessed("Using first fit CheckOut-Strategy");
              // implement a CheckOut-Strategy by using a freight with the
              // given amount
              break;
      }
      default: System.err.println("No strategy");
    }
    switch(strategyTower) {
      case MainFrame.STRATEGY_TOWER_LAST_POS: {
              task.fireTaskProcessed("Tower stays at last position");
              // do nothing... tower stays where it is.
              break;
      }
      case MainFrame.STRATEGY_TOWER_FAVORITE_POS: {
              task.fireTaskProcessed("Tower moves to a strategic position");
              // get a strategic position of the tower using the frequency Count.
              break;
      }
    }
    /** @todo IMPLEMENT THIS FEATURE */

    task.fireTaskTerminated("after a commit and when the driver responded the task is terminated");
  }

}// end of class