package infosys;

import java.util.*;
import de.web.databasesupport.tools.database.*;
import java.sql.*;
import javax.swing.DefaultComboBoxModel;

/** The Control is the central point of this application.
 *  It provides the database connection as well as methods
 *  to access program representations of check in/out units.
 *  This Control acts is Scheduler as well as a TaskListener, that
 *  is added by default to all Unit constructed CheckIn/ CheckOut-Tasks.
 */
public class Control implements Scheduler, TaskListener {

  protected static Database db = null;

  private Dispatcher dispatcher = new Dispatcher(5000, this);

  private Vector catalogItems = new Vector(100);

  /** Models for the caalog comboboxes. */
  protected Vector cms = new Vector(2);

  /** Model for unit lists. */
  protected javax.swing.DefaultComboBoxModel cmbmUnit = null;

  /** Vector to store all units constructed out of database information. */
  protected Vector units;

  final static String SQL_UPDATE_UNIT = "UPDATE Ramp SET name = ?, posX = ?, posY = ? WHERE rampId = ?; ";
  static PreparedStatement pre_UPDATE_UNIT = null;

  final static String SQL_SELECT_UNIT = "SELECT rampId, name, posX, posY, Tin, Tout FROM Ramp; ";
  static PreparedStatement pre_SELECT_UNIT = null;

  final static String SQL_INSERT_UNIT = "INSERT INTO Ramp (name, posX, posY, Tin, Tout) VALUES (?,?,?,?,?); ";
  static PreparedStatement pre_INSERT_UNIT = null;

  final static String SQL_EXITS_UNIT = "SELECT rampId FROM Ramp WHERE posX = ? AND posY = ?; ";
  static PreparedStatement pre_EXISTS_UNIT = null;

  final static String SQL_DELETE_UNIT = "DELETE * FROM Ramp WHERE rampId = ?; ";
  static PreparedStatement pre_DELETE_UNIT = null;


  final static String SQL_UPDATE_CATALOG = "UPDATE Catalog_Freight SET name = ? WHERE name = ?; ";
  static PreparedStatement pre_UPDATE_CATALOG = null;

  final static String SQL_SELECT_CATALOG = "SELECT name FROM Catalog_Freight; ";
  static PreparedStatement pre_SELECT_CATALOG = null;

  final static String SQL_INSERT_CATALOG = "INSERT INTO Catalog_Freight (name) VALUES (?); ";
  static PreparedStatement pre_INSERT_CATALOG = null;

  final static String SQL_EXITS_CATALOG = "SELECT name FROM Catalog_Freight WHERE name = ? SORT BY name";
  static PreparedStatement pre_EXISTS_CATALOG = null;

  final static String SQL_DELETE_CATALOG = "DELETE * FROM Catalog_Freight WHERE name = ?; ";
  static PreparedStatement pre_DELETE_CATALOG = null;



  /** Protected Constructor for this class. */
  public Control(Database db) {
    setDB(db);
    try {
        sqlInit();
        myInit();
    } catch (Exception exc) {
        exc.printStackTrace();
    }
  }

  public Dispatcher getDispatcher() { return this.dispatcher; }

  private void sqlInit() throws Exception {
    if (db != null && db.isClosed()) db.open();
    pre_UPDATE_UNIT = db.precompile(SQL_UPDATE_UNIT);
    pre_SELECT_UNIT = db.precompile(SQL_SELECT_UNIT);
    pre_EXISTS_UNIT = db.precompile(SQL_EXITS_UNIT);
    pre_INSERT_UNIT = db.precompile(SQL_INSERT_UNIT);
    pre_DELETE_UNIT = db.precompile(SQL_DELETE_UNIT);

    pre_UPDATE_CATALOG = db.precompile(SQL_UPDATE_CATALOG);
    pre_SELECT_CATALOG = db.precompile(SQL_SELECT_CATALOG);
    pre_EXISTS_CATALOG = db.precompile(SQL_EXITS_CATALOG);
    pre_INSERT_CATALOG = db.precompile(SQL_INSERT_CATALOG);
    pre_DELETE_CATALOG = db.precompile(SQL_DELETE_CATALOG);

  }

  private void myInit() throws Exception {
    createUnits();
    createCatalog();
  }

  protected synchronized void createCatalog() {
    if (db == null) throw new RuntimeException("No database specified");

    try {
        ResultSet r = pre_SELECT_CATALOG.executeQuery();

        while (r.next()) {
            catalogItems.add(r.getString(1));
        }

    } catch (Exception exc) {
        exc.printStackTrace();
    }
  }

  /** Creates Unit Objects from the Database information. */
  protected synchronized void createUnits() {
    if (db == null) throw new RuntimeException("No database specified");

    // create dummy data....
    // @todo implement adequate SQL Statements.
    units = new java.util.Vector(3);
    int id, x, y;
    String n;
    int in, out;
    ResultSet r;

    try {
        r = pre_SELECT_UNIT.executeQuery();

        while (r.next()) {
            id = r.getInt(1);
            n = r.getString(2);
            x = r.getInt(3);
            y = r.getInt(4);
            in = (r.getBoolean(5))? Unit.TYPE_CHECK_IN: 0;
            out =(r.getBoolean(6))? Unit.TYPE_CHECK_OUT: 0;
            units.add(new Unit(n,x,y,in | out,this,id));
        }
        r.close();
    } catch (Exception exc) {
        exc.printStackTrace();
    }
    cmbmUnit = new javax.swing.DefaultComboBoxModel(units.toArray());
  }

  public synchronized void addToCatalog(String catalogEntry) {
    if (db == null) throw new RuntimeException("No database specified");

    try {
        pre_INSERT_CATALOG.clearParameters();
        pre_INSERT_CATALOG.setString(1, catalogEntry);

        pre_INSERT_CATALOG.executeUpdate();
        db.commit();

        for (int i=0; i<cms.size(); i++ ) {
            DefaultComboBoxModel m = (DefaultComboBoxModel) cms.get(i);
            m.addElement(catalogEntry);
        }
    } catch (Exception exc) {
        exc.printStackTrace();
    }
  }

  public synchronized void addToUnits(String name, int x, int y, int type) {
    if (db == null) throw new RuntimeException("No database specified");
    int id;
    ResultSet rs, rs2;

    try {
        pre_EXISTS_UNIT.setInt(1,x);
        pre_EXISTS_UNIT.setInt(2,y);
        rs = pre_EXISTS_UNIT.executeQuery();
        // already ocupied position.
        if (rs.next()) {
            rs.close();
            return;
        }
        rs.close();
        pre_INSERT_UNIT.clearParameters();
        pre_INSERT_UNIT.setString(1,name);
        pre_INSERT_UNIT.setInt(2,x);
        pre_INSERT_UNIT.setInt(3,y);
        pre_INSERT_UNIT.setBoolean(4, (type & Control.Unit.TYPE_CHECK_IN) == Control.Unit.TYPE_CHECK_IN);
        pre_INSERT_UNIT.setBoolean(5, (type & Control.Unit.TYPE_CHECK_OUT) == Control.Unit.TYPE_CHECK_OUT);

        pre_INSERT_UNIT.executeUpdate();
        db.commit();

        pre_EXISTS_UNIT.clearParameters();
        pre_EXISTS_UNIT.setInt(1,x);
        pre_EXISTS_UNIT.setInt(2,y);
        rs2 = pre_EXISTS_UNIT.executeQuery();
        // already ocupied position.
        if (rs2.next()) {
            id = rs2.getInt(1);
            Unit u = new Unit(name, x, y, type, this, id);
            units.add(u);
            cmbmUnit.addElement(u);
        }
        rs2.close();
    }
    catch (SQLException sexc ) {
        sexc.printStackTrace();
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
  }

  public synchronized void deleteCatalog(String name) {
    if (db == null) throw new RuntimeException("No database specified");

    try {
        pre_DELETE_CATALOG.setString(1,name);
        pre_DELETE_CATALOG.executeUpdate();
        db.commit();

        for (int i=0; i<cms.size(); i++) {
            DefaultComboBoxModel m = (DefaultComboBoxModel) cms.get(i);
            m.removeElement(name);
        }

    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
  }


  public synchronized void deleteUnit(Unit u) {
    if (db == null) throw new RuntimeException("No database specified");

    try {
        pre_DELETE_UNIT.setInt(1,u.getId());
        pre_DELETE_UNIT.executeUpdate();
        db.commit();
        units.removeElement(u);
        cmbmUnit.removeElement(u);
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
  }

  public synchronized void changeCatalog(String nameOld, String nameNew) {
    if (db == null) throw new RuntimeException("No database specified");

    try {
        pre_UPDATE_CATALOG.setString(1,nameNew);
        pre_UPDATE_CATALOG.setString(2,nameOld);

        pre_UPDATE_CATALOG.executeUpdate();
        db.commit();

        for (int i=0; i<cms.size(); i++) {
            DefaultComboBoxModel m = (DefaultComboBoxModel) cms.get(i);
            m.removeElement(nameOld);
            m.addElement(nameNew);
        }

    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
  }

  public synchronized void changeUnit(Unit u, String name, int x, int y) {
    if (db == null) throw new RuntimeException("No database specified");

    try {
        pre_UPDATE_UNIT.setString(1,name);
        pre_UPDATE_UNIT.setInt(   2,x);
        pre_UPDATE_UNIT.setInt(   3,y);
        pre_UPDATE_UNIT.setInt(   4,u.getId());

        pre_UPDATE_UNIT.executeUpdate();
        db.commit();

        u.name = name;
        u.x    = x;
        u.y    = y;
    }
    catch (Exception ex) {
        ex.printStackTrace();
    }
  }


  public javax.swing.ComboBoxModel createCBM() {
    javax.swing.DefaultComboBoxModel m = new javax.swing.DefaultComboBoxModel(catalogItems);
    return m;
  }

  /** Call only once per combobox */
  public javax.swing.ComboBoxModel getCatalogModel() {
    javax.swing.ComboBoxModel m = createCBM();
    cms.add(m);
    return m;
  }

  /** Returns the model for the units. */
  public javax.swing.ComboBoxModel getUnitsModel() {
    return cmbmUnit;
  }


  /** Returns all Unit objects that are constructed out of database information.
   *  @return all Units.
   */
  public Unit[] getUnits() {
    Unit[] rv = new Unit[units.size()];
    for (int i=0; i<rv.length; i++) {
        rv[i] = (Unit) units.get(i);
    }
    return rv;
  }

  public Database getDB() {
    return db;
  }

  protected void setDB(Database db) {
    this.db = db;
    /** @todo uncomment next line when db functionality is impl. */
    // createUnits();
  }

  /** Represants a checkin/ checkout unit constructed out of database information.
   *  A unit can have different types: TYPE_CHECK_IN, TYPE_CHECK_OUT or both.
   *  A graphical representation might want to use the checkIn() or checkOut() method
   *  of this member class.
   */
  public class Unit {

    /** Specifies a check in unit. */
    public final static int TYPE_CHECK_IN  = 1;

    /** Specifies a check out unit. */
    public final static int TYPE_CHECK_OUT = 2;

    /** The name, or identifier of this unit. */
    protected String name;

    /** The logical x-position in high rack storage coordinates. */
    protected int x;

    /** The logical y-position in high rack storage coordinates. */
    protected int y;

    /** The type of the unit. Either TYPE_CHECK_IN, TYPE_CHECK_OUT or both. */
    protected int type;

    /** The Control object that constructes this unit. */
    protected Control control;

    protected int id = Integer.MIN_VALUE;

    /** Constructs a new Unit.
     *  @param name    The (human readable) identifier of this unit.
     *  @param x       The x position in high rack storage coordinates.
     *  @param y       The y position in high rack storage coordinates.
     *  @param type    The type of this unit (TYPE_CHECK_IN and/or TYPE_CHECK_OUT).
     *  @param control The Control that constructs this object.
     */
    protected Unit(String name, int x, int y, int type, Control control, int id) {
      this.name = name;
      this.x = x;
      this.y = y;
      this.type = type;
      this.control = control;
      this.id = id;
    }

    public java.util.Vector tl_list = new java.util.Vector(1);


    public String  getName() { return name; }
    public int     getX() { return x; }
    public int     getY() { return y; }
    public int     getType() { return type; }
    public Control getControl() { return control; }
    public int     getId() { return id; }

    public void addTaskListener(TaskListener tl) {
        tl_list.add(tl);
    }

    public void removeTaskListener(TaskListener tl) {
        tl_list.remove(tl);
    }


    /** Checks in a freight.
     *  @param freightName The catalog name (see DB) of the freight.
     *  @param amount      The amount of the freight.
     *  @param expiration  The date when the freight expires.
     */
    public Task checkIn(String freightName, long amount, java.sql.Date expiration) {
      Task task = new CheckInTask (this, freightName, amount, expiration);
      task.addTaskListener(control);
      for (int i=0; i <tl_list.size(); i++) {
        TaskListener t = (TaskListener) tl_list.get(i);
        task.addTaskListener(t);
      }
      control.schedule(task);
      return task;
    }

    /** Checks out a freight.
     *  @param freightName The catalog name of the freight.
     *  @param amoount     The amount to check out.
     */
    public Task checkOut(String freightName, long amount) {
      Task task = new CheckOutTask (this, freightName, amount);
      task.addTaskListener(control);
      for (int i=0; i <tl_list.size(); i++) {
        TaskListener t = (TaskListener) tl_list.get(i);
        task.addTaskListener(t);
      }
      control.schedule(task);
      return task;
    }

    public String toString() {
        return this.getName();
    }

  }

/*----------------------------------------------------------------------------*/
/*                                                   Scheduler implementation */

  protected java.util.Vector queue = new java.util.Vector();
  protected Task idleTask = new Task() {};

  public void schedule(Task task) {
    queue.add(task);
    task.fireTaskScheduled("control has scheduled task.");
  }

  public Task getTask() {
    Task rv;
    if (queue.size() > 0) {
        rv = (Task) queue.elementAt(0);
        queue.remove(0);
    } else {
        rv = idleTask;
    }

    return rv;
  }


/*----------------------------------------------------------------------------*/
/*                                                TaskListener implementation */

  public void taskScheduled(TaskEvent te) {}
  public void taskProcessed(TaskEvent te) {}
  public void taskTerminated(TaskEvent te) {}
  public void taskInterrupted(TaskExceptionEvent tee) {}

} // end of class.