package infosys;

/** A TaskEvent is used to commit data to TaskListeners.
 *  When constructed the current sytem time is stored in the time field.
 */
public class TaskEvent extends java.util.EventObject {

  /** The system time when the TaskEvent was constructed. */
  protected long time;

  /** The message for this TaskEvent. */
  protected String message = "no message available for this TaskEvent";

  /** Constructs a new TaskEvent.
   *  @param source The event source, usually a Task.
   */
  public TaskEvent (Object source) {
    super(source);
    time = System.currentTimeMillis();
  }

  /** Constructs a new TaskEvent.
   *  @param source  The event source, usually a Task.
   *  @param message The message for this Task.
   */
  public TaskEvent (Object source, String message) {
    this (source);
    this.message = message;
  }

  /** Returns the creation time for this TaskEvent.
   *  The creation time depends on the system it was created.
   *  Be aware of this when operation in multi system environments.
   *  @return The creation time in millis.
   */
  public long getTime() {
    return this.time;
  }

  /** Human readable description of this Event.
   *  Contains time and message.
   *  @return A human readable description.
   */
  public String toString() {
    return "TaskEvent @ " + time + " " + getMessage();
  }

  /** Returns the message of this TaskEvent.
   *  @return The message.
   */
  public String getMessage() {
    return this.message;
  }

}// end of class.