package infosys;

/** Special TaskEvent that is used to report exceptional conditions of the Task.
 *  A Exception is related with this EventObject, so that a TaskListener can be
 *  informed of any exceptional state of a Task.
 */
public class TaskExceptionEvent extends TaskEvent {

  /** The related Exception. */
  protected Exception exception = null;

  /** Constructs a new TaskExceptionEvent.
   *  @param source The source of the event, usually a Task object.
   *  @param exc    The related Exception.
   */
  public TaskExceptionEvent (Object source, Exception exc) {
    super(source);
    this.exception = exc;
  }

  /** Constructs a new TaskExceptionEvent.
   *  @param source  The source of the event, usually a Task object.
   *  @param message The message for this TaskExceptionEvent.
   *  @param exc     The related Exception.
   */
  public TaskExceptionEvent (Object source, String message, Exception exc) {
    super(source, message);
    this.exception = exc;
  }

  /** Returns the related Exception.
   *  @return The Exception.
   */
  public Exception getException() {
    return this.exception;
  }

}// end of task